/*!
FullCalendar Resource Day Grid Plugin v7.0.0-beta.4
Docs & License: https://fullcalendar.io/docs/resource-daygrid-view
(c) 2024 Adam Shaw
*/
FullCalendar.ResourceDayGrid = (function (exports, core, premiumCommonPlugin, resourcePlugin, dayGridPlugin, internal$3, preact, internal$2, internal$1) {
    'use strict';

    function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

    var premiumCommonPlugin__default = /*#__PURE__*/_interopDefault(premiumCommonPlugin);
    var resourcePlugin__default = /*#__PURE__*/_interopDefault(resourcePlugin);
    var dayGridPlugin__default = /*#__PURE__*/_interopDefault(dayGridPlugin);

    class ResourceDayTableJoiner extends internal$1.VResourceJoiner {
        transformSeg(seg, resourceDayTableModel, resourceI) {
            let colRanges = resourceDayTableModel.computeColRanges(seg.start, seg.end, resourceI);
            return colRanges.map((colRange) => (Object.assign(Object.assign(Object.assign({}, seg), colRange), { isStart: seg.isStart && colRange.isStart, isEnd: seg.isEnd && colRange.isEnd })));
        }
    }

    function buildResourceRowConfigs(resources, datesAboveResources, dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, // TODO: rename to dateHeaderFormat?
    context) {
        if (!resources.length) {
            return [
                internal$2.buildDateRowConfig(dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, context)
            ];
        }
        if (dates.length === 1) {
            return [
                buildResourceRowConfig(resources, undefined, context)
            ];
        }
        if (datesAboveResources) {
            const resourceDataConfigsPerDate = dates.map((date) => {
                return buildResourceDataConfigs(resources, date, context);
            });
            return [
                internal$2.buildDateRowConfig(dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, context, 
                /* colSpan = */ resources.length),
                {
                    renderConfig: buildResourceRenderConfig(context),
                    dataConfigs: [].concat(...resourceDataConfigsPerDate), // flatten
                }
            ];
        }
        else {
            const dateDataConfigsPerResource = resources.map((resource) => {
                return internal$2.buildDateDataConfigs(dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, context, 1, // rowSpan
                resource.id + ':');
            });
            return [
                buildResourceRowConfig(resources, undefined, context, /* colSpan = */ dates.length),
                {
                    renderConfig: internal$2.buildDateRenderConfig(context),
                    dataConfigs: [].concat(...dateDataConfigsPerResource), // flatten
                }
            ];
        }
    }
    function buildResourceRowConfig(resources, date, context, colSpan) {
        return {
            renderConfig: buildResourceRenderConfig(context),
            dataConfigs: buildResourceDataConfigs(resources, date, context, colSpan),
        };
    }
    function buildResourceRenderConfig(context) {
        const { options } = context;
        return {
            generatorName: 'resourceLabelContent',
            customGenerator: options.resourceLabelContent,
            classNameGenerator: options.resourceLabelClassNames,
            didMount: options.resourceLabelDidMount,
            willUnmount: options.resourceLabelWillUnmount,
        };
    }
    function buildResourceDataConfigs(resources, date, context, colSpan = 1) {
        return resources.map((resource) => {
            const resourceApi = new resourcePlugin.ResourceApi(context, resource);
            const resourceApiId = resourceApi.id;
            return {
                key: (date ? date.toUTCString() + ':' : '') + resource.id,
                renderProps: {
                    resource: resourceApi,
                    text: resource.title || resourceApiId || '',
                    isDisabled: false,
                    date: date ? context.dateEnv.toDate(date) : null,
                    view: context.viewApi,
                },
                attrs: {
                    // will these NOT render if values are undefined? I hope so
                    'data-resource-id': resourceApiId,
                    'data-date': date ? internal$3.formatDayString(date) : undefined,
                },
                colSpan,
            };
        });
    }

    class ResourceDayGridView extends internal$3.DateComponent {
        constructor() {
            super(...arguments);
            // memo
            this.flattenResources = internal$3.memoize(internal$1.flattenResources);
            this.buildResourceDayTableModel = internal$3.memoize(buildResourceDayTableModel);
            this.createDayHeaderFormatter = internal$3.memoize(internal$2.createDayHeaderFormatter);
            this.buildResourceRowConfigs = internal$3.memoize(buildResourceRowConfigs);
            this.splitter = new internal$1.VResourceSplitter();
            this.slicers = {};
            this.joiner = new ResourceDayTableJoiner();
            this.isHitComboAllowed = (hit0, hit1) => {
                let allowAcrossResources = this.resourceDayTableModel.dayTableModel.colCnt === 1;
                return allowAcrossResources || hit0.dateSpan.resourceId === hit1.dateSpan.resourceId;
            };
        }
        render() {
            let { props, context } = this;
            let { options } = context;
            let resourceOrderSpecs = options.resourceOrder || internal$1.DEFAULT_RESOURCE_ORDER;
            let resources = this.flattenResources(props.resourceStore, resourceOrderSpecs);
            let resourceDayTableModel = this.resourceDayTableModel = this.buildResourceDayTableModel(props.dateProfile, context.dateProfileGenerator, resources, options.datesAboveResources, context);
            let splitProps = this.splitter.splitProps({
                resourceDayTableModel,
                businessHours: props.businessHours,
                dateSelection: props.dateSelection,
                eventStore: props.eventStore,
                eventUiBases: props.eventUiBases,
                eventSelection: props.eventSelection,
                eventDrag: props.eventDrag,
                eventResize: props.eventResize,
            });
            this.slicers = internal$3.mapHash(splitProps, (split, resourceId) => this.slicers[resourceId] || new internal$2.DayTableSlicer());
            let slicedProps = internal$3.mapHash(this.slicers, (slicer, resourceId) => slicer.sliceProps(splitProps[resourceId], props.dateProfile, options.nextDayThreshold, context, resourceDayTableModel.dayTableModel));
            let joinedSlicedProps = this.joiner.joinProps(slicedProps, resourceDayTableModel);
            let datesRepDistinctDays = resourceDayTableModel.dayTableModel.rowCnt === 1;
            let dayHeaderFormat = this.createDayHeaderFormatter(context.options.dayHeaderFormat, datesRepDistinctDays, resourceDayTableModel.colCnt);
            return (preact.createElement(internal$3.NowTimer, { unit: "day" }, (nowDate, todayRange) => {
                const headerTiers = this.buildResourceRowConfigs(resources, options.datesAboveResources, resourceDayTableModel.dayTableModel.headerDates, datesRepDistinctDays, props.dateProfile, todayRange, dayHeaderFormat, context);
                return (preact.createElement(internal$2.DayGridLayout, { labelId: props.labelId, labelStr: props.labelStr, dateProfile: props.dateProfile, todayRange: todayRange, cellRows: resourceDayTableModel.cells, forPrint: props.forPrint, isHitComboAllowed: this.isHitComboAllowed, className: 'fc-resource-daygrid-view', 
                    // header content
                    headerTiers: headerTiers, 
                    // body content
                    fgEventSegs: joinedSlicedProps.fgEventSegs, bgEventSegs: joinedSlicedProps.bgEventSegs, businessHourSegs: joinedSlicedProps.businessHourSegs, dateSelectionSegs: joinedSlicedProps.dateSelectionSegs, eventDrag: joinedSlicedProps.eventDrag, eventResize: joinedSlicedProps.eventResize, eventSelection: joinedSlicedProps.eventSelection }));
            }));
        }
    }
    function buildResourceDayTableModel(dateProfile, dateProfileGenerator, resources, datesAboveResources, context) {
        let dayTable = internal$2.buildDayTableModel(dateProfile, dateProfileGenerator);
        if (!resources.length) {
            return new internal$1.ResourcelessDayTableModel(dayTable);
        }
        return datesAboveResources ?
            new internal$1.DayResourceTableModel(dayTable, resources, context) :
            new internal$1.ResourceDayTableModel(dayTable, resources, context);
    }

    var plugin = core.createPlugin({
        name: '@fullcalendar/resource-daygrid',
        premiumReleaseDate: '2025-01-09',
        deps: [
            premiumCommonPlugin__default["default"],
            resourcePlugin__default["default"],
            dayGridPlugin__default["default"],
        ],
        initialView: 'resourceDayGridDay',
        views: {
            resourceDayGrid: {
                type: 'dayGrid',
                component: ResourceDayGridView,
                needsResourceData: true,
            },
            resourceDayGridDay: {
                type: 'resourceDayGrid',
                duration: { days: 1 },
            },
            resourceDayGridWeek: {
                type: 'resourceDayGrid',
                duration: { weeks: 1 },
            },
            resourceDayGridMonth: {
                type: 'resourceDayGrid',
                duration: { months: 1 },
                fixedWeekCount: true,
            },
        },
    });

    var internal = {
        __proto__: null,
        ResourceDayTableJoiner: ResourceDayTableJoiner,
        ResourceDayGridView: ResourceDayGridView,
        buildResourceRowConfigs: buildResourceRowConfigs
    };

    core.globalPlugins.push(plugin);

    exports.Internal = internal;
    exports["default"] = plugin;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, FullCalendar, FullCalendar.PremiumCommon, FullCalendar.Resource, FullCalendar.DayGrid, FullCalendar.Internal, FullCalendar.Preact, FullCalendar.DayGrid.Internal, FullCalendar.Resource.Internal);
