'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var internal_cjs = require('@fullcalendar/resource/internal.cjs');
var internal_cjs$2 = require('@fullcalendar/core/internal.cjs');
var preact_cjs = require('@fullcalendar/core/preact.cjs');
var internal_cjs$1 = require('@fullcalendar/daygrid/internal.cjs');
var resourcePlugin = require('@fullcalendar/resource/index.cjs');

class ResourceDayTableJoiner extends internal_cjs.VResourceJoiner {
    transformSeg(seg, resourceDayTableModel, resourceI) {
        let colRanges = resourceDayTableModel.computeColRanges(seg.start, seg.end, resourceI);
        return colRanges.map((colRange) => (Object.assign(Object.assign(Object.assign({}, seg), colRange), { isStart: seg.isStart && colRange.isStart, isEnd: seg.isEnd && colRange.isEnd })));
    }
}

function buildResourceRowConfigs(resources, datesAboveResources, dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, // TODO: rename to dateHeaderFormat?
context) {
    if (!resources.length) {
        return [
            internal_cjs$1.buildDateRowConfig(dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, context)
        ];
    }
    if (dates.length === 1) {
        return [
            buildResourceRowConfig(resources, undefined, context)
        ];
    }
    if (datesAboveResources) {
        const resourceDataConfigsPerDate = dates.map((date) => {
            return buildResourceDataConfigs(resources, date, context);
        });
        return [
            internal_cjs$1.buildDateRowConfig(dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, context, 
            /* colSpan = */ resources.length),
            {
                renderConfig: buildResourceRenderConfig(context),
                dataConfigs: [].concat(...resourceDataConfigsPerDate), // flatten
            }
        ];
    }
    else {
        const dateDataConfigsPerResource = resources.map((resource) => {
            return internal_cjs$1.buildDateDataConfigs(dates, datesRepDistinctDays, dateProfile, todayRange, dayHeaderFormat, context, 1, // rowSpan
            resource.id + ':');
        });
        return [
            buildResourceRowConfig(resources, undefined, context, /* colSpan = */ dates.length),
            {
                renderConfig: internal_cjs$1.buildDateRenderConfig(context),
                dataConfigs: [].concat(...dateDataConfigsPerResource), // flatten
            }
        ];
    }
}
function buildResourceRowConfig(resources, date, context, colSpan) {
    return {
        renderConfig: buildResourceRenderConfig(context),
        dataConfigs: buildResourceDataConfigs(resources, date, context, colSpan),
    };
}
function buildResourceRenderConfig(context) {
    const { options } = context;
    return {
        generatorName: 'resourceLabelContent',
        customGenerator: options.resourceLabelContent,
        classNameGenerator: options.resourceLabelClassNames,
        didMount: options.resourceLabelDidMount,
        willUnmount: options.resourceLabelWillUnmount,
    };
}
function buildResourceDataConfigs(resources, date, context, colSpan = 1) {
    return resources.map((resource) => {
        const resourceApi = new resourcePlugin.ResourceApi(context, resource);
        const resourceApiId = resourceApi.id;
        return {
            key: (date ? date.toUTCString() + ':' : '') + resource.id,
            renderProps: {
                resource: resourceApi,
                text: resource.title || resourceApiId || '',
                isDisabled: false,
                date: date ? context.dateEnv.toDate(date) : null,
                view: context.viewApi,
            },
            attrs: {
                // will these NOT render if values are undefined? I hope so
                'data-resource-id': resourceApiId,
                'data-date': date ? internal_cjs$2.formatDayString(date) : undefined,
            },
            colSpan,
        };
    });
}

class ResourceDayGridView extends internal_cjs$2.DateComponent {
    constructor() {
        super(...arguments);
        // memo
        this.flattenResources = internal_cjs$2.memoize(internal_cjs.flattenResources);
        this.buildResourceDayTableModel = internal_cjs$2.memoize(buildResourceDayTableModel);
        this.createDayHeaderFormatter = internal_cjs$2.memoize(internal_cjs$1.createDayHeaderFormatter);
        this.buildResourceRowConfigs = internal_cjs$2.memoize(buildResourceRowConfigs);
        this.splitter = new internal_cjs.VResourceSplitter();
        this.slicers = {};
        this.joiner = new ResourceDayTableJoiner();
        this.isHitComboAllowed = (hit0, hit1) => {
            let allowAcrossResources = this.resourceDayTableModel.dayTableModel.colCnt === 1;
            return allowAcrossResources || hit0.dateSpan.resourceId === hit1.dateSpan.resourceId;
        };
    }
    render() {
        let { props, context } = this;
        let { options } = context;
        let resourceOrderSpecs = options.resourceOrder || internal_cjs.DEFAULT_RESOURCE_ORDER;
        let resources = this.flattenResources(props.resourceStore, resourceOrderSpecs);
        let resourceDayTableModel = this.resourceDayTableModel = this.buildResourceDayTableModel(props.dateProfile, context.dateProfileGenerator, resources, options.datesAboveResources, context);
        let splitProps = this.splitter.splitProps({
            resourceDayTableModel,
            businessHours: props.businessHours,
            dateSelection: props.dateSelection,
            eventStore: props.eventStore,
            eventUiBases: props.eventUiBases,
            eventSelection: props.eventSelection,
            eventDrag: props.eventDrag,
            eventResize: props.eventResize,
        });
        this.slicers = internal_cjs$2.mapHash(splitProps, (split, resourceId) => this.slicers[resourceId] || new internal_cjs$1.DayTableSlicer());
        let slicedProps = internal_cjs$2.mapHash(this.slicers, (slicer, resourceId) => slicer.sliceProps(splitProps[resourceId], props.dateProfile, options.nextDayThreshold, context, resourceDayTableModel.dayTableModel));
        let joinedSlicedProps = this.joiner.joinProps(slicedProps, resourceDayTableModel);
        let datesRepDistinctDays = resourceDayTableModel.dayTableModel.rowCnt === 1;
        let dayHeaderFormat = this.createDayHeaderFormatter(context.options.dayHeaderFormat, datesRepDistinctDays, resourceDayTableModel.colCnt);
        return (preact_cjs.createElement(internal_cjs$2.NowTimer, { unit: "day" }, (nowDate, todayRange) => {
            const headerTiers = this.buildResourceRowConfigs(resources, options.datesAboveResources, resourceDayTableModel.dayTableModel.headerDates, datesRepDistinctDays, props.dateProfile, todayRange, dayHeaderFormat, context);
            return (preact_cjs.createElement(internal_cjs$1.DayGridLayout, { labelId: props.labelId, labelStr: props.labelStr, dateProfile: props.dateProfile, todayRange: todayRange, cellRows: resourceDayTableModel.cells, forPrint: props.forPrint, isHitComboAllowed: this.isHitComboAllowed, className: 'fc-resource-daygrid-view', 
                // header content
                headerTiers: headerTiers, 
                // body content
                fgEventSegs: joinedSlicedProps.fgEventSegs, bgEventSegs: joinedSlicedProps.bgEventSegs, businessHourSegs: joinedSlicedProps.businessHourSegs, dateSelectionSegs: joinedSlicedProps.dateSelectionSegs, eventDrag: joinedSlicedProps.eventDrag, eventResize: joinedSlicedProps.eventResize, eventSelection: joinedSlicedProps.eventSelection }));
        }));
    }
}
function buildResourceDayTableModel(dateProfile, dateProfileGenerator, resources, datesAboveResources, context) {
    let dayTable = internal_cjs$1.buildDayTableModel(dateProfile, dateProfileGenerator);
    if (!resources.length) {
        return new internal_cjs.ResourcelessDayTableModel(dayTable);
    }
    return datesAboveResources ?
        new internal_cjs.DayResourceTableModel(dayTable, resources, context) :
        new internal_cjs.ResourceDayTableModel(dayTable, resources, context);
}

exports.ResourceDayGridView = ResourceDayGridView;
exports.ResourceDayTableJoiner = ResourceDayTableJoiner;
exports.buildResourceRowConfigs = buildResourceRowConfigs;
